/* generic.c

Copyright (C) GraphiCode, Inc 1992 All Rights Reserved.

Generic aperture list converter - use as a template for writing your own.
You don't need our permission to do this IF IT IS FOR YOUR OWN USE.
Commercial applications require written permission from GraphiCode.

Produces an ASCII file suitable for reading by GraphiCode products.
This particular implementation converts Barco/Disc .WHE files.

Assumes Microsoft C for the _dos_findfirst and _dos_findnext functions.
These are also implemented for POSIX (#define unix 1). 

Comments and questions can be directed to the SYSOP of the GraphiCode 
Tech Support BBS at (206)771-6980.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#ifdef unix
#define PATH_SEP_CHAR '/'
#define strupr(c) c
#include <dirent.h>
DIR *dirp;
struct dirent *direntp;
struct find_t {
  char name[80];
  };
int _dos_findfirst(char *path,int mode,struct find_t *c_file);
int _dos_findnext(struct find_t *c_file);
#define _A_NORMAL 0
#else /* DOS */
#include <dos.h>
#define PATH_SEP_CHAR '\\'
#endif
#define DRIVE_DESIG_CHAR ':'

/* aperture types */
#define A_DRAW 0   /* DRAW and ROUND are equivalent on a laser plotter */
#define A_ROUND 1  /* ROUND is for flash, DRAW for draw, on vector plotters */
#define A_OCTAGON 2
#define A_SQUARE 3
#define A_RECTANGLE 4
#define A_OBLONG 5
#define A_DONUT 6
#define A_THERMAL 7
#define A_TARGET 8
#define A_CUSTOM 9
#define A_SLIT 10
#define A_CUSTOMTHERMAL 11 /* just a custom, but known to be a thermal */
#define A_POEX 12 /* external polygon */
#define A_POIN 12 /* internal polygon */

enum UNIT_TYPES {INCH,MM,MIL};

char *cpaUnitNames[] = {"\"","mm","mil"};
char *cpaFullUnitNames[] = {"INCH","MM","MIL"};
char *cpaAperNames[] = {
  "Draw     ","Round    ","Octagon  ","Square   ","Rectangle",
  "Oblong   ","Annulus  ","Thermal  ","Target   ","Custom   ",
  "Slit     ","Cstm Thrm","Ext. Poly","Int. Poly"};

char *cpDefaultInfileExt = ".WHE";
char *cpDefaultOutfileExt = ".APR";

int main(int argc, char *argv[]); /* proto for quiet compile */

int main(argc,argv)
  int argc;
  char *argv[];
  {
  FILE *rep_file;
  FILE *apr_file;
  char infilebuf[120]; /* input buffer */
  char outfilebuf[120]; /* output buffer */
  char CustName[120]; /* for construct custom aperture names */
  char *cptr;
  struct find_t c_file;
  int units,n_apers = 0,Dcode,apertype;
  double width,height,corner;

  /* announce ourselves */
  printf("GENERIC v1.0 - Generic GraphiCode Aperture Wheel Converter\n");
  printf("Copyright (C) GraphiCode, Inc. 1992.  All Rights Reserved.\n");
  if ((argc < 2) || (argc > 3)) {
    printf("usage: GENERIC <aperture file> [<outfile>]\n\n");
    printf("Example:\n  To convert the aperture file \"ALIST1.WHE\" ");
    printf("for use with GraphiCode products, type:\n    GENERIC ALIST1\n");
    printf("This creates the GraphiCode aperture wheel file \"ALIST1.APR\".\n");
    printf("To convert \"ALIST1.WHE\" to \"NEWALIST.APR\" type:\n");
    printf("    GENERIC ALIST1 NEWALIST\n");
    printf("To convert several aperture files type:\n    GENERIC *.WHE\n");
    exit(1);
    }

  strcpy(infilebuf,argv[1]); /* get input filename */

  if (!strchr(infilebuf,'.')) /* no extension - add default */
    strcpy(infilebuf+strlen(infilebuf),cpDefaultInfileExt);

  /* find first file that matches mask */
  _dos_findfirst(infilebuf,_A_NORMAL,&c_file);

  /* now loop on all files which match the mask */
  do {
    if (cptr=strrchr(infilebuf,PATH_SEP_CHAR))
      cptr++; /* add fname to path */
    else if (cptr = strrchr(infilebuf,DRIVE_DESIG_CHAR))
      cptr++; /* add fname to path */
    else
      cptr = infilebuf; /* not a path */
    strcpy(cptr,c_file.name);
    if ((rep_file = fopen(infilebuf,"r")) == NULL) {
      printf("Failed to open input file \"%s\"",infilebuf);
      exit(1);
      }
    if (argc==3)  /* output name differs from input */
      strcpy(outfilebuf,strupr(argv[2]));
    else /* reuse the first arg */
      strcpy(outfilebuf,infilebuf);
    if (!strrchr(outfilebuf,'.')) /* 8-16-91 BSP */
      strcpy(outfilebuf+strlen(outfilebuf),cpDefaultOutfileExt);
    else
      strcpy(strrchr(outfilebuf,'.'),cpDefaultOutfileExt);
    if ((apr_file = fopen(outfilebuf,"w"))==NULL) {
      printf("Failed to open output file \"%s\"",outfilebuf);
      exit(2);
      }

    /* here we go */
    printf("\nConverting aperture file \"%s\" to \"%s\"... ",infilebuf,outfilebuf);

    /* write header */
    fprintf(apr_file,"\n\n	 %s\n	 Pos Drl R90 Code   Type      Dimensions\n",
        outfilebuf);

    /* Strip off Header Information, get units */
    while (!feof(rep_file)) {
      fgets(infilebuf,120,rep_file);
      if (!strncmp(infilebuf,";; Aperture Unit",16)) { /* is this start of header? */
        fgets(infilebuf,120,rep_file); /* get Units line */
        if (!strncmp(infilebuf,"INCH",4))
          units = INCH;
        else if (!strncmp(infilebuf,"MM",2))
          units = MM;
        else 
          units = MIL;
        }
      else if (!strncmp(infilebuf,";; Apertures",12)) /* start of apertures */
        break; /* done searching */
      }

    /* read the apertures */
    while (fgets(infilebuf,120,rep_file)) { /* next line */
      if (!strncmp(infilebuf,"APERTURE(",9)) { /* a .WHE aper description? */
        n_apers++; /* found one! */
        Dcode = atoi(strchr(infilebuf,'(')+1); /* read D code */

        /* Find aperture shape.  These are DPF shapes. */
        cptr = strchr(infilebuf,'=')+1;
        if (!strncmp(cptr,"CON",2)) {
          apertype = A_POEX;
          width = height = 0.0;
          }
        else if (!strncmp(cptr,"CIR",1)) {
          apertype = A_ROUND;
          width = height = atof(cptr = (strchr(cptr,',')+1));
          if (cptr = strchr(cptr,',')) { /* a second arg - an annulus */
            height = atof(cptr+1);
            apertype = A_DONUT;
            }
          }
        else if (!strncmp(cptr,"DON",1)) {
          apertype = A_DONUT;
          width = atof(cptr = (strchr(cptr,',')+1));
          height = atof(strchr(cptr,',')+1);
          }
        else if (!strncmp(cptr,"BOX",2)) {
          width = atof(cptr = (strchr(cptr,',')+1));
          height = atof(cptr = (strchr(cptr,',')+1));
          cptr = (strchr(cptr,',')+1); /* find corner type */
          if (*cptr=='R') { /* round rect */
            apertype = A_CUSTOM;
            corner = atof((strchr(cptr,',')+1)); /* find corner radius */
            sprintf(CustName,"=RR,%f,%f,%f,%s",
              width,height,corner,cpaFullUnitNames[units]);
            }
          else if ((*cptr=='S') && (height==width)) /* assume octagon */
            apertype = A_OCTAGON;
          else { /* unknown shape */
            apertype = A_CUSTOM;
            strcpy(CustName,"triangle.cap");
            }
          }
        else if (!strncmp(cptr,"SQU",1)) {
          apertype = A_SQUARE;
          width = height = atof(strchr(cptr,',')+1);
          }
        else if (!strncmp(cptr,"REC",1)) {
          apertype = A_RECTANGLE;
          width = atof(cptr = (strchr(cptr,',')+1));
          height = atof(strchr(cptr,',')+1);
          }
        else { /* unknown shape */
          apertype = A_CUSTOM;
          strcpy(CustName,"triangle.cap");
          }

        /* Write the aperture description. Don't mess with this section! */
        if ((apertype == A_CUSTOM) || (apertype == A_CUSTOMTHERMAL))
          fprintf(apr_file,"	%3d T0 %3d D%-3d %s %s\n",
            n_apers,n_apers,Dcode,cpaAperNames[apertype],CustName);
        else if ((apertype<=A_SQUARE)||(apertype==A_SLIT)) /* 1-D */
          fprintf(apr_file,"	%3d T0 %3d D%-3d %s %f %s\n",
            n_apers,n_apers,Dcode,cpaAperNames[apertype],width,
              cpaUnitNames[units]);
        else if ((apertype==A_POEX)||(apertype==A_POIN)) /* 0-D */
          fprintf(apr_file,"	%3d T0 %3d D%-3d %s\n",
            n_apers,n_apers,Dcode,cpaAperNames[apertype]);
        else                                              /* 2-D */
          fprintf(apr_file,"	%3d T0 %3d D%-3d %s %f %s x %f %s\n",
            n_apers,n_apers,Dcode,cpaAperNames[apertype],
              width,cpaUnitNames[units],height,cpaUnitNames[units]);
        } /* end if it is an aperture description */
      } /* End while not feof of aperture file */
    fclose(rep_file); /* done with this file */
    fclose(apr_file); /* done with this file */
    printf("  found %d apertures.\n",n_apers);
    } while (!_dos_findnext(&c_file)); /* any more files to convert? */
  return(0); /* all done, all OK */
  } /* end main() */

#ifdef unix
int _dos_findfirst(char *path,int mode,struct find_t *c_file) {
  mode++; /* for quiet compile */
  if (!(dirp = opendir(path)))
    return(1);
  if (direntp = readdir(dirp)) {
    strcpy(c_file->name,direntp->d_name);
    return(0);
    }
  else
    return(1);
  }

int _dos_findnext(struct find_t *c_file) {
  if (direntp = readdir(dirp)) {
    strcpy(c_file->name,direntp->d_name);
    return(0);
    }
  else {
    closedir(dirp);
    return(1);
    }
  }
#endif
